@@ -6,15 +6,15 @@
 #include "qstringlist.h"
 #include "qfile.h"
 #if QT_CONFIG(settings)
+#include "qresource.h"
 #include "qsettings.h"
 #endif
 #include "qlibraryinfo.h"
 #include "qlibraryinfo_p.h"
-#include "qscopedpointer.h"
 
 #include "qcoreapplication.h"
 
-#include "private/qglobal_p.h"
+#include "private/qfilesystementry_p.h"
 #include "archdetect.cpp"
 #include "qconfig.cpp"
 
@@ -30,6 +30,8 @@
 #    include <qt_windows.h>
 #endif
 
+#include <memory>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -38,7 +40,7 @@ extern void qDumpCPUFeatures(); // in qsimd.cpp
 
 #if QT_CONFIG(settings)
 
-static QSettings *findConfiguration();
+static std::unique_ptr<QSettings> findConfiguration();
 
 struct QLibrarySettings
 {
@@ -47,7 +49,7 @@ struct QLibrarySettings
     bool havePaths();
     QSettings *configuration();
 
-    QScopedPointer<QSettings> settings;
+    std::unique_ptr<QSettings> settings;
     bool paths;
     bool reloadOnQAppAvailable;
 };
@@ -62,7 +64,7 @@ QSettings *QLibrarySettings::configuration()
 {
     if (reloadOnQAppAvailable && QCoreApplication::instance() != nullptr)
         load();
-    return settings.data();
+    return settings.get();
 }
 
 bool QLibrarySettings::havePaths()
@@ -75,8 +77,8 @@ bool QLibrarySettings::havePaths()
 void QLibrarySettings::load()
 {
     // If we get any settings here, those won't change when the application shows up.
-    settings.reset(findConfiguration());
-    reloadOnQAppAvailable = (settings.data() == nullptr && QCoreApplication::instance() == nullptr);
+    settings = findConfiguration();
+    reloadOnQAppAvailable = !settings && !QCoreApplication::instance();
 
     if (settings) {
         // This code needs to be in the regular library, as otherwise a qt.conf that
@@ -96,14 +98,14 @@ void QLibraryInfoPrivate::setQtconfManualPath(const QString *path)
     qtconfManualPath = path;
 }
 
-static QSettings *findConfiguration()
+static std::unique_ptr<QSettings> findConfiguration()
 {
     if (qtconfManualPath)
-        return new QSettings(*qtconfManualPath, QSettings::IniFormat);
+        return std::make_unique<QSettings>(*qtconfManualPath, QSettings::IniFormat);
 
     QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
-    if (QFile::exists(qtconfig))
-        return new QSettings(qtconfig, QSettings::IniFormat);
+    if (QResource(qtconfig, QLocale::c()).isValid())
+        return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
 #ifdef Q_OS_DARWIN
     CFBundleRef bundleRef = CFBundleGetMainBundle();
     if (bundleRef) {
@@ -115,18 +117,18 @@ static QSettings *findConfiguration()
             QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
             qtconfig = QDir::cleanPath(path);
             if (QFile::exists(qtconfig))
-                return new QSettings(qtconfig, QSettings::IniFormat);
+                return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
         }
     }
 #endif
     if (QCoreApplication::instance()) {
-        QDir pwd(QCoreApplication::applicationDirPath());
-        qtconfig = pwd.filePath(u"qt" QT_STRINGIFY(QT_VERSION_MAJOR) ".conf"_s);
+        QString pwd = QCoreApplication::applicationDirPath();
+        qtconfig = pwd + u"/qt" QT_STRINGIFY(QT_VERSION_MAJOR) ".conf"_s;
         if (QFile::exists(qtconfig))
-            return new QSettings(qtconfig, QSettings::IniFormat);
-        qtconfig = pwd.filePath("qt.conf"_L1);
+            return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
+        qtconfig = pwd + u"/qt.conf";
         if (QFile::exists(qtconfig))
-            return new QSettings(qtconfig, QSettings::IniFormat);
+            return std::make_unique<QSettings>(qtconfig, QSettings::IniFormat);
     }
     return nullptr;     //no luck
 }
@@ -268,8 +270,6 @@ QVersionNumber QLibraryInfo::version() noexcept
 
 static QString prefixFromAppDirHelper()
 {
-    QString appDir;
-
     if (QCoreApplication::instance()) {
 #ifdef Q_OS_DARWIN
         CFBundleRef bundleRef = CFBundleGetMainBundle();
@@ -288,12 +288,10 @@ static QString prefixFromAppDirHelper()
         }
 #endif // Q_OS_DARWIN
         // We make the prefix path absolute to the executable's directory.
-        appDir = QCoreApplication::applicationDirPath();
+        return QCoreApplication::applicationDirPath();
     } else {
-        appDir = QDir::currentPath();
+        return QDir::currentPath();
     }
-
-    return appDir;
 }
 
 #if QT_CONFIG(relocatable)
@@ -516,12 +514,169 @@ QLibraryInfoPrivate::LocationInfo QLibraryInfoPrivate::locationInfo(QLibraryInfo
 /*!
     \since 6.0
     Returns the path specified by \a p.
+
+    If there is more than one path listed in qt.conf, it will
+    only return the first one.
+    \sa paths
 */
 QString QLibraryInfo::path(LibraryPath p)
 {
     return QLibraryInfoPrivate::path(p);
 }
 
+/*!
+    \since 6.8
+    Returns all paths specificied by \a p.
+
+    \sa path
+ */
+QStringList QLibraryInfo::paths(LibraryPath p)
+{
+    return QLibraryInfoPrivate::paths(p);
+}
+
+static bool keepQtBuildDefaults()
+{
+#if QT_CONFIG(settings)
+    QSettings *config = QLibraryInfoPrivate::configuration();
+    Q_ASSERT(config != nullptr);
+    return config->value("Config/MergeQtConf", false).toBool();
+#else
+    return false;
+#endif
+}
+
+#if QT_CONFIG(settings)
+static QString normalizePath(QString ret)
+{
+    qsizetype startIndex = 0;
+    /* We support placeholders of the form $(<ENV_VAR>) in qt.conf.
+       The loop below tries to find all such placeholders, and replaces
+       them with the actual value of the ENV_VAR environment variable
+     */
+    while (true) {
+        startIndex = ret.indexOf(u'$', startIndex);
+        if (startIndex < 0)
+            break;
+        if (ret.size() < startIndex + 3)
+            break;
+        if (ret.at(startIndex + 1) != u'(') {
+            startIndex++;
+            continue;
+        }
+        qsizetype endIndex = ret.indexOf(u')', startIndex + 2);
+        if (endIndex < 0)
+            break;
+        auto envVarName = QStringView{ret}.sliced(startIndex + 2, endIndex - startIndex - 2);
+        QString value = qEnvironmentVariable(envVarName.toLocal8Bit().constData());
+        ret.replace(startIndex, endIndex - startIndex + 1, value);
+        startIndex += value.size();
+    }
+    return QDir::fromNativeSeparators(ret);
+};
+
+static QVariant libraryPathToValue(QLibraryInfo::LibraryPath loc)
+{
+    QVariant value;
+    auto li = QLibraryInfoPrivate::locationInfo(loc);
+    if (li.key.isNull())
+        return value;
+    QSettings *config = QLibraryInfoPrivate::configuration();
+    Q_ASSERT(config != nullptr);
+    // if keepQtBuildDefaults returns true,
+    // we only consider explicit values listed in qt.conf
+    QVariant defaultValue = keepQtBuildDefaults()
+            ? QVariant()
+            : QVariant(li.defaultValue);
+    config->beginGroup("Paths"_L1);
+    auto cleanup = qScopeGuard([&]() { config->endGroup(); });
+    if (li.fallbackKey.isNull()) {
+        value = config->value(li.key, defaultValue);
+    } else {
+        value = config->value(li.key);
+        if (!value.isValid())
+            value = config->value(li.fallbackKey, defaultValue);
+    }
+    return value;
+}
+#endif // settings
+
+// TODO: There apparently are paths that are both absolute and relative for QFileSystemEntry.
+//       In particular on windows.
+
+static bool pathIsRelative(const QString &path)
+{
+    using FromInternalPath = QFileSystemEntry::FromInternalPath;
+    return !path.startsWith(':'_L1) && QFileSystemEntry(path, FromInternalPath{}).isRelative();
+}
+
+static bool pathIsAbsolute(const QString &path)
+{
+    using FromInternalPath = QFileSystemEntry::FromInternalPath;
+    return path.startsWith(':'_L1) || QFileSystemEntry(path, FromInternalPath{}).isAbsolute();
+}
+
+QStringList QLibraryInfoPrivate::paths(QLibraryInfo::LibraryPath p,
+                                       UsageMode usageMode)
+{
+    const QLibraryInfo::LibraryPath loc = p;
+    QList<QString> ret;
+    bool fromConf = false;
+    bool pathsAreAbsolute = true;
+#if QT_CONFIG(settings)
+    if (havePaths()) {
+        fromConf = true;
+
+        QVariant value = libraryPathToValue(loc);
+        if (value.isValid()) {
+            if (auto *asList = get_if<QList<QString>>(&value))
+                ret = std::move(*asList);
+            else
+                ret = QList<QString>({ std::move(value).toString()});
+            for (qsizetype i = 0, end = ret.size(); i < end; ++i) {
+                ret[i] = normalizePath(ret[i]);
+                pathsAreAbsolute = pathsAreAbsolute && pathIsAbsolute(ret[i]);
+            }
+        }
+    }
+#endif // settings
+
+    if (!fromConf || keepQtBuildDefaults()) {
+        QString noConfResult;
+        if (loc == QLibraryInfo::PrefixPath) {
+            noConfResult = getPrefix(usageMode);
+        } else if (int(loc) <= qt_configure_strs.count()) {
+            noConfResult = QString::fromLocal8Bit(qt_configure_strs.viewAt(loc - 1));
+#ifndef Q_OS_WIN // On Windows we use the registry
+        } else if (loc == QLibraryInfo::SettingsPath) {
+            // Use of volatile is a hack to discourage compilers from calling
+            // strlen(), in the inlined fromLocal8Bit(const char *)'s body, at
+            // compile-time, as Qt installers binary-patch the path, replacing
+            // the dummy path seen at compile-time, typically changing length.
+            const char *volatile path = QT_CONFIGURE_SETTINGS_PATH;
+            noConfResult = QString::fromLocal8Bit(path);
+#endif
+        }
+        if (!noConfResult.isEmpty()) {
+            pathsAreAbsolute = pathsAreAbsolute && pathIsAbsolute(noConfResult);
+            ret.push_back(std::move(noConfResult));
+        }
+    }
+    if (ret.isEmpty() || pathsAreAbsolute)
+        return ret;
+
+    QString baseDir;
+    if (loc == QLibraryInfo::PrefixPath) {
+        baseDir = prefixFromAppDirHelper();
+    } else {
+        // we make any other path absolute to the prefix directory
+        baseDir = QLibraryInfoPrivate::path(QLibraryInfo::PrefixPath, usageMode);
+    }
+    for (qsizetype i = 0, end = ret.size(); i < end; ++i)
+        if (pathIsRelative(ret[i]))
+            ret[i] = QDir::cleanPath(baseDir + u'/' +  std::move(ret[i]));
+    return ret;
+}
 
 /*
     Returns the path specified by \a p.
@@ -531,83 +686,7 @@ QString QLibraryInfo::path(LibraryPath p)
  */
 QString QLibraryInfoPrivate::path(QLibraryInfo::LibraryPath p, UsageMode usageMode)
 {
-    const QLibraryInfo::LibraryPath loc = p;
-    QString ret;
-    bool fromConf = false;
-#if QT_CONFIG(settings)
-    if (havePaths()) {
-        fromConf = true;
-
-        auto li = QLibraryInfoPrivate::locationInfo(loc);
-        if (!li.key.isNull()) {
-            QSettings *config = QLibraryInfoPrivate::configuration();
-            Q_ASSERT(config != nullptr);
-            config->beginGroup("Paths"_L1);
-
-            if (li.fallbackKey.isNull()) {
-                ret = config->value(li.key, li.defaultValue).toString();
-            } else {
-                QVariant v = config->value(li.key);
-                if (!v.isValid())
-                    v = config->value(li.fallbackKey, li.defaultValue);
-                ret = v.toString();
-            }
-
-            qsizetype startIndex = 0;
-            while (true) {
-                startIndex = ret.indexOf(u'$', startIndex);
-                if (startIndex < 0)
-                    break;
-                if (ret.size() < startIndex + 3)
-                    break;
-                if (ret.at(startIndex + 1) != u'(') {
-                    startIndex++;
-                    continue;
-                }
-                qsizetype endIndex = ret.indexOf(u')', startIndex + 2);
-                if (endIndex < 0)
-                    break;
-                auto envVarName = QStringView{ret}.mid(startIndex + 2, endIndex - startIndex - 2);
-                QString value = QString::fromLocal8Bit(qgetenv(envVarName.toLocal8Bit().constData()));
-                ret.replace(startIndex, endIndex - startIndex + 1, value);
-                startIndex += value.size();
-            }
-
-            config->endGroup();
-
-            ret = QDir::fromNativeSeparators(ret);
-        }
-    }
-#endif // settings
-
-    if (!fromConf) {
-        if (loc == QLibraryInfo::PrefixPath) {
-            ret = getPrefix(usageMode);
-        } else if (int(loc) <= qt_configure_strs.count()) {
-            ret = QString::fromLocal8Bit(qt_configure_strs.viewAt(loc - 1));
-#ifndef Q_OS_WIN // On Windows we use the registry
-        } else if (loc == QLibraryInfo::SettingsPath) {
-            // Use of volatile is a hack to discourage compilers from calling
-            // strlen(), in the inlined fromLocal8Bit(const char *)'s body, at
-            // compile-time, as Qt installers binary-patch the path, replacing
-            // the dummy path seen at compile-time, typically changing length.
-            const char *volatile path = QT_CONFIGURE_SETTINGS_PATH;
-            ret = QString::fromLocal8Bit(path);
-#endif
-        }
-    }
-
-    if (!ret.isEmpty() && QDir::isRelativePath(ret)) {
-        QString baseDir;
-        if (loc == QLibraryInfo::PrefixPath) {
-            baseDir = prefixFromAppDirHelper();
-        } else {
-            // we make any other path absolute to the prefix directory
-            baseDir = path(QLibraryInfo::PrefixPath, usageMode);
-        }
-        ret = QDir::cleanPath(baseDir + u'/' + ret);
-    }
-    return ret;
+    return paths(p, usageMode).value(0, QString());
 }
 
 /*!
@@ -626,8 +705,7 @@ QString QLibraryInfoPrivate::path(QLibraryInfo::LibraryPath p, UsageMode usageMo
 QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
 {
 #if QT_CONFIG(settings)
-    QScopedPointer<const QSettings> settings(findConfiguration());
-    if (!settings.isNull()) {
+    if (const auto settings = findConfiguration()) {
         const QString key = "Platforms/"_L1
                 + platformName
                 + "Arguments"_L1;
@@ -667,7 +745,7 @@ QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
 
 /*!
     \typealias QLibraryInfo::LibraryLocation
-    \deprecated Use LibraryPath with QLibraryInfo::path() instead.
+    \deprecated [6.0] Use LibraryPath with QLibraryInfo::path() instead.
 */
 
 /*!
