@@ -1,46 +1,14 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qioseventdispatcher.h"
 #include "qiosapplicationdelegate.h"
 #include "qiosglobal.h"
 
+#if defined(Q_OS_VISIONOS)
+#include "qiosswiftintegration.h"
+#endif
+
 #include <QtCore/qprocessordetection.h>
 #include <QtCore/private/qcoreapplication_p.h>
 #include <QtCore/private/qthread_p.h>
@@ -204,10 +172,21 @@ namespace
     jmp_buf applicationWillTerminateJumpPoint;
 
     bool debugStackUsage = false;
+
+    struct {
+        QAppleLogActivity UIApplicationMain;
+        QAppleLogActivity applicationDidFinishLaunching;
+    } logActivity;
+
+    static bool s_isQtApplication = false;
 }
 
+using namespace QT_PREPEND_NAMESPACE(QtPrivate);
+
 extern "C" int qt_main_wrapper(int argc, char *argv[])
 {
+    s_isQtApplication = true;
+
     @autoreleasepool {
         size_t defaultStackSize = 512 * kBytesPerKiloByte; // Same as secondary threads
 
@@ -228,8 +207,19 @@ extern "C" int qt_main_wrapper(int argc, char *argv[])
             }
         }
 
-        qEventDispatcherDebug() << "Running UIApplicationMain"; qIndent();
+        logActivity.UIApplicationMain = QT_APPLE_LOG_ACTIVITY(
+            lcEventDispatcher().isDebugEnabled(), "UIApplicationMain").enter();
+
+#if defined(Q_OS_VISIONOS)
+        Q_UNUSED(argc);
+        Q_UNUSED(argv);
+        qCDebug(lcEventDispatcher) << "Starting Swift app";
+        QIOSIntegrationPluginSwift::runSwiftAppMain();
+        Q_UNREACHABLE();
+#else
+        qCDebug(lcEventDispatcher) << "Running UIApplicationMain";
         return UIApplicationMain(argc, argv, nil, NSStringFromClass([QIOSApplicationDelegate class]));
+#endif
     }
 }
 
@@ -245,7 +235,7 @@ extern "C" int main(int argc, char *argv[]);
 
 static void __attribute__((noinline, noreturn)) user_main_trampoline()
 {
-    NSArray *arguments = [[NSProcessInfo processInfo] arguments];
+    NSArray<NSString *> *arguments = [[NSProcessInfo processInfo] arguments];
     int argc = arguments.count;
     char **argv = new char*[argc];
 
@@ -263,11 +253,14 @@ static void __attribute__((noinline, noreturn)) user_main_trampoline()
     int exitCode = main(argc, argv);
     delete[] argv;
 
-    qEventDispatcherDebug() << "Returned from main with exit code " << exitCode;
+    logActivity.applicationDidFinishLaunching.enter();
+    qCDebug(lcEventDispatcher) << "Returned from main with exit code " << exitCode;
 
     if (Q_UNLIKELY(debugStackUsage))
         userMainStack.printUsage();
 
+    logActivity.applicationDidFinishLaunching.leave();
+
     if (applicationAboutToTerminate)
         longjmp(applicationWillTerminateJumpPoint, kJumpedFromUserMainTrampoline);
 
@@ -294,7 +287,7 @@ static bool rootLevelRunLoopIntegration()
 {
     [[NSNotificationCenter defaultCenter]
         addObserver:self
-        selector:@selector(applicationDidFinishLaunching)
+        selector:@selector(applicationDidFinishLaunching:)
         name:UIApplicationDidFinishLaunchingNotification
         object:nil];
 
@@ -320,8 +313,13 @@ static bool rootLevelRunLoopIntegration()
 #  error "Unknown processor family"
 #endif
 
-+ (void)applicationDidFinishLaunching
++ (void)applicationDidFinishLaunching:(NSNotification *)notification
 {
+    logActivity.applicationDidFinishLaunching = QT_APPLE_LOG_ACTIVITY_WITH_PARENT(
+        lcEventDispatcher().isDebugEnabled(), "applicationDidFinishLaunching", logActivity.UIApplicationMain).enter();
+
+    qCDebug(lcEventDispatcher) << "Application launched with options" << notification.userInfo;
+
     if (!isQtApplication())
         return;
 
@@ -329,7 +327,7 @@ static bool rootLevelRunLoopIntegration()
         // We schedule the main-redirection for the next run-loop pass, so that we
         // can return from this function and let UIApplicationMain finish its job.
         // This results in running Qt's application eventloop as a nested runloop.
-        qEventDispatcherDebug() << "Scheduling main() on next run-loop pass";
+        qCDebug(lcEventDispatcher) << "Scheduling main() on next run-loop pass";
         CFRunLoopTimerRef userMainTimer = CFRunLoopTimerCreateWithHandler(kCFAllocatorDefault,
              CFAbsoluteTimeGetCurrent(), 0, 0, 0, ^(CFRunLoopTimerRef) { user_main_trampoline(); });
         CFRunLoopAddTimer(CFRunLoopGetMain(), userMainTimer, kCFRunLoopCommonModes);
@@ -338,8 +336,9 @@ static bool rootLevelRunLoopIntegration()
     }
 
     switch (setjmp(processEventEnterJumpPoint)) {
-    case kJumpPointSetSuccessfully:
-        qEventDispatcherDebug() << "Running main() on separate stack"; qIndent();
+    case kJumpPointSetSuccessfully: {
+        qCDebug(lcEventDispatcher) << "Running main() on separate stack";
+        QT_APPLE_SCOPED_LOG_ACTIVITY(lcEventDispatcher().isDebugEnabled(), "main()");
 
         // Redirect the stack pointer to the start of the reserved stack. This ensures
         // that when we longjmp out of the event dispatcher and continue execution, the
@@ -355,10 +354,12 @@ static bool rootLevelRunLoopIntegration()
 
         Q_UNREACHABLE();
         break;
+    }
     case kJumpedFromEventDispatcherProcessEvents:
         // We've returned from the longjmp in the event dispatcher,
         // and the stack has been restored to its old self.
-        qUnIndent(); qEventDispatcherDebug() << "Returned from processEvents";
+        logActivity.UIApplicationMain.enter();
+        qCDebug(lcEventDispatcher) << "↳ Jumped from processEvents due to exec";
 
         if (Q_UNLIKELY(debugStackUsage))
             userMainStack.printUsage();
@@ -376,6 +377,10 @@ static const char kApplicationWillTerminateExitCode = char(SIGTERM | 0x80);
 
 + (void)applicationWillTerminate
 {
+    QAppleLogActivity applicationWillTerminateActivity = QT_APPLE_LOG_ACTIVITY_WITH_PARENT(
+        lcEventDispatcher().isDebugEnabled(), "applicationWillTerminate", logActivity.UIApplicationMain).enter();
+    qCDebug(lcEventDispatcher) << "Application about to be terminated by iOS";
+
     if (!isQtApplication())
         return;
 
@@ -394,18 +399,21 @@ static const char kApplicationWillTerminateExitCode = char(SIGTERM | 0x80);
     applicationAboutToTerminate = true;
     switch (setjmp(applicationWillTerminateJumpPoint)) {
     case kJumpPointSetSuccessfully:
-        qEventDispatcherDebug() << "Exiting qApp with SIGTERM exit code"; qIndent();
+        qCDebug(lcEventDispatcher) << "Exiting qApp with SIGTERM exit code";
         qApp->exit(kApplicationWillTerminateExitCode);
 
         // The runloop will not exit when the application is about to terminate,
         // so we'll never see the exit activity and have a chance to return from
         // QEventLoop::exec(). We initiate the return manually as a workaround.
-        qEventDispatcherDebug() << "Manually triggering return from event loop exec";
-        static_cast<QIOSEventDispatcher *>(qApp->eventDispatcher())->interruptEventLoopExec();
+        qCDebug(lcEventDispatcher) << "Manually triggering return from event loop exec";
+        applicationWillTerminateActivity.leave();
+        static_cast<QIOSJumpingEventDispatcher *>(qApp->eventDispatcher())->interruptEventLoopExec();
         break;
     case kJumpedFromUserMainTrampoline:
+        applicationWillTerminateActivity.enter();
         // The user's main has returned, so we're ready to let iOS terminate the application
-        qUnIndent(); qEventDispatcherDebug() << "kJumpedFromUserMainTrampoline, allowing iOS to terminate";
+        qCDebug(lcEventDispatcher) << "kJumpedFromUserMainTrampoline, allowing iOS to terminate";
+        applicationWillTerminateActivity.leave();
         break;
     default:
         qFatal("Unexpected jump result in event loop integration");
@@ -417,26 +425,67 @@ static const char kApplicationWillTerminateExitCode = char(SIGTERM | 0x80);
 QT_BEGIN_NAMESPACE
 QT_USE_NAMESPACE
 
+QIOSEventDispatcher *QIOSEventDispatcher::create()
+{
+    if (isQtApplication() && rootLevelRunLoopIntegration())
+        return new QIOSJumpingEventDispatcher;
+
+    return new QIOSEventDispatcher;
+}
+
 QIOSEventDispatcher::QIOSEventDispatcher(QObject *parent)
     : QEventDispatcherCoreFoundation(parent)
+{
+    // We want all delivery of events from the system to be handled synchronously
+    QWindowSystemInterface::setSynchronousWindowSystemEvents(true);
+}
+
+bool QIOSEventDispatcher::isQtApplication()
+{
+    return s_isQtApplication;
+}
+
+/*!
+    Override of the CoreFoundation posted events runloop source callback
+    so that we can send window system (QPA) events in addition to sending
+    normal Qt events.
+*/
+bool QIOSEventDispatcher::processPostedEvents()
+{
+    // Don't send window system events if the base CF dispatcher has determined
+    // that events should not be sent for this pass of the runloop source.
+    if (!QEventDispatcherCoreFoundation::processPostedEvents())
+        return false;
+
+    QT_APPLE_SCOPED_LOG_ACTIVITY(lcEventDispatcher().isDebugEnabled(), "sendWindowSystemEvents");
+    QEventLoop::ProcessEventsFlags flags
+        = QEventLoop::ProcessEventsFlags(m_processEvents.flags.loadRelaxed());
+    qCDebug(lcEventDispatcher) << "Sending window system events for" << flags;
+    QWindowSystemInterface::sendWindowSystemEvents(flags);
+
+    return true;
+}
+
+QIOSJumpingEventDispatcher::QIOSJumpingEventDispatcher(QObject *parent)
+    : QIOSEventDispatcher(parent)
     , m_processEventLevel(0)
-    , m_runLoopExitObserver(this, &QIOSEventDispatcher::handleRunLoopExit, kCFRunLoopExit)
+    , m_runLoopExitObserver(this, &QIOSJumpingEventDispatcher::handleRunLoopExit, kCFRunLoopExit)
 {
 }
 
-bool __attribute__((returns_twice)) QIOSEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
+bool __attribute__((returns_twice)) QIOSJumpingEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
 {
-    if (!rootLevelRunLoopIntegration())
-        return QEventDispatcherCoreFoundation::processEvents(flags);
-
     if (applicationAboutToTerminate) {
-        qEventDispatcherDebug() << "Detected QEventLoop exec after application termination";
+        qCDebug(lcEventDispatcher) << "Detected QEventLoop exec after application termination";
         // Re-issue exit, and return immediately
         qApp->exit(kApplicationWillTerminateExitCode);
         return false;
     }
 
     if (!m_processEventLevel && (flags & QEventLoop::EventLoopExec)) {
+        QT_APPLE_SCOPED_LOG_ACTIVITY(lcEventDispatcher().isDebugEnabled(), "processEvents");
+        qCDebug(lcEventDispatcher) << "Processing events with flags" << flags;
+
         ++m_processEventLevel;
 
         m_runLoopExitObserver.addToMode(kCFRunLoopCommonModes);
@@ -445,7 +494,7 @@ bool __attribute__((returns_twice)) QIOSEventDispatcher::processEvents(QEventLoo
         // is asked to exit, so that we can return from QEventLoop::exec().
         switch (setjmp(processEventExitJumpPoint)) {
         case kJumpPointSetSuccessfully:
-            qEventDispatcherDebug() << "QEventLoop exec detected, jumping back to native runloop";
+            qCDebug(lcEventDispatcher) << "QEventLoop exec detected, jumping back to system runloop ↵";
             longjmp(processEventEnterJumpPoint, kJumpedFromEventDispatcherProcessEvents);
             break;
         case kJumpedFromEventLoopExecInterrupt:
@@ -453,7 +502,7 @@ bool __attribute__((returns_twice)) QIOSEventDispatcher::processEvents(QEventLoo
             // signal), and we jumped back though processEventExitJumpPoint. We return from processEvents,
             // which will emit aboutToQuit if it's QApplication's event loop, and then return to the user's
             // main, which can do whatever it wants, including calling exec() on the application again.
-            qEventDispatcherDebug() << "kJumpedFromEventLoopExecInterrupt, returning with eventsProcessed = true";
+            qCDebug(lcEventDispatcher) << "⇢ System runloop exited, returning with eventsProcessed = true";
             return true;
         default:
             qFatal("Unexpected jump result in event loop integration");
@@ -469,37 +518,16 @@ bool __attribute__((returns_twice)) QIOSEventDispatcher::processEvents(QEventLoo
     return processedEvents;
 }
 
-/*!
-    Override of the CoreFoundation posted events runloop source callback
-    so that we can send window system (QPA) events in addition to sending
-    normal Qt events.
-*/
-bool QIOSEventDispatcher::processPostedEvents()
-{
-    // Don't send window system events if the base CF dispatcher has determined
-    // that events should not be sent for this pass of the runloop source.
-    if (!QEventDispatcherCoreFoundation::processPostedEvents())
-        return false;
-
-    qEventDispatcherDebug() << "Sending window system events for " << m_processEvents.flags; qIndent();
-    QWindowSystemInterface::sendWindowSystemEvents(m_processEvents.flags);
-    qUnIndent();
-
-    return true;
-}
-
-void QIOSEventDispatcher::handleRunLoopExit(CFRunLoopActivity activity)
+void QIOSJumpingEventDispatcher::handleRunLoopExit(CFRunLoopActivity activity)
 {
     Q_UNUSED(activity);
     Q_ASSERT(activity == kCFRunLoopExit);
 
-    if (m_processEventLevel == 1 && !currentEventLoop()->isRunning()) {
-        qEventDispatcherDebug() << "Root runloop level exited";
+    if (m_processEventLevel == 1 && !currentEventLoop()->isRunning())
         interruptEventLoopExec();
-    }
 }
 
-void QIOSEventDispatcher::interruptEventLoopExec()
+void QIOSJumpingEventDispatcher::interruptEventLoopExec()
 {
     Q_ASSERT(m_processEventLevel == 1);
 
@@ -512,12 +540,14 @@ void QIOSEventDispatcher::interruptEventLoopExec()
     // processEvents, instead of back in didFinishLaunchingWithOptions.
     switch (setjmp(processEventEnterJumpPoint)) {
     case kJumpPointSetSuccessfully:
-        qEventDispatcherDebug() << "Jumping back to processEvents";
+        qCDebug(lcEventDispatcher) << "Jumping into processEvents due to system runloop exit ⇢";
+        logActivity.UIApplicationMain.leave();
         longjmp(processEventExitJumpPoint, kJumpedFromEventLoopExecInterrupt);
         break;
     case kJumpedFromEventDispatcherProcessEvents:
         // QEventLoop was re-executed
-        qEventDispatcherDebug() << "kJumpedFromEventDispatcherProcessEvents";
+        logActivity.UIApplicationMain.enter();
+        qCDebug(lcEventDispatcher) << "↳ Jumped from processEvents due to re-exec";
         break;
     default:
         qFatal("Unexpected jump result in event loop integration");
