@@ -191,6 +191,10 @@ bool QQnxScreenEventHandler::handleEvent(screen_event_t event, int qnxType)
         handlePropertyEvent(event);
         break;
 
+    case SCREEN_EVENT_MANAGER:
+        handleManagerEvent(event);
+        break;
+
     default:
         // event ignored
         qCDebug(lcQpaScreenEvents) << Q_FUNC_INFO << "Unknown event" << qnxType;
@@ -606,11 +610,35 @@ void QQnxScreenEventHandler::handleCloseEvent(screen_event_t event)
 void QQnxScreenEventHandler::handleCreateEvent(screen_event_t event)
 {
     screen_window_t window = 0;
+    int object_type = -1;
+
     Q_SCREEN_CHECKERROR(
+        screen_get_event_property_iv(event, SCREEN_PROPERTY_OBJECT_TYPE, &object_type),
+        "Failed to query object type for create event");
+
+    switch (object_type) {
+    // Other object types than window produces an unnessary warning, thus ignore 
+    case SCREEN_OBJECT_TYPE_CONTEXT:
+    case SCREEN_OBJECT_TYPE_GROUP:
+    case SCREEN_OBJECT_TYPE_DISPLAY:
+    case SCREEN_OBJECT_TYPE_DEVICE:
+    case SCREEN_OBJECT_TYPE_PIXMAP:
+    case SCREEN_OBJECT_TYPE_SESSION:
+    case SCREEN_OBJECT_TYPE_STREAM:
+        break;
+    case SCREEN_OBJECT_TYPE_WINDOW:
+    {
+        Q_SCREEN_CHECKERROR(
             screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, (void**)&window),
             "Failed to query window property");
 
-    Q_EMIT newWindowCreated(window);
+        Q_EMIT newWindowCreated(window);
+        break;
+    }
+    default:
+        qCDebug(lcQpaScreenEvents) << "Ignore create event for object type: " << object_type;
+        break;
+    }
 }
 
 void QQnxScreenEventHandler::handleDisplayEvent(screen_event_t event)
@@ -673,6 +701,11 @@ void QQnxScreenEventHandler::handlePropertyEvent(screen_event_t event)
     if (Q_UNLIKELY(screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, (void**)&window) != 0))
         qFatal("QQnx: failed to query window property, errno=%d", errno);
 
+    if (window == 0) {
+        qCDebug(lcQpaScreenEvents) << "handlePositionEvent on NULL window";
+        return;
+    }
+
     errno = 0;
     int property;
     if (Q_UNLIKELY(screen_get_event_property_iv(event, SCREEN_PROPERTY_NAME, &property) != 0))
@@ -747,4 +780,30 @@ void QQnxScreenEventHandler::timerEvent(QTimerEvent *event)
 
 QT_END_NAMESPACE
 
+void QQnxScreenEventHandler::handleManagerEvent(screen_event_t event)
+{
+    errno = 0;
+    int subtype;
+    Q_SCREEN_CHECKERROR(
+            screen_get_event_property_iv(event, SCREEN_PROPERTY_SUBTYPE, &subtype),
+            "Failed to query object type property");
+
+    errno = 0;
+    screen_window_t window = 0;
+    if (screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, (void**)&window) != 0)
+        qFatal("QQnx: failed to query window property, errno=%d", errno);
+
+    switch (subtype) {
+    case SCREEN_EVENT_CLOSE: {
+        QWindow *closeWindow = QQnxIntegration::instance()->window(window);
+        closeWindow->close();
+        break;
+    }
+
+    default:
+        // event ignored
+        qCDebug(lcQpaScreenEvents) << "Ignore manager event for subtype: " << subtype;
+    }
+}
+
 #include "moc_qqnxscreeneventhandler.cpp"
