@@ -30,6 +30,8 @@ static bool initResources()
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::Literals::StringLiterals;
+
 /*!
     \class QWindowsCursorCacheKey
     \brief Cache key for storing values in a QHash with a QCursor as key.
@@ -103,14 +105,16 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
         hotSpot.setX(width / 2);
     if (hotSpot.y() < 0)
         hotSpot.setY(height / 2);
-    const int n = qMax(1, width / 8);
-    QScopedArrayPointer<uchar> xBits(new uchar[height * n]);
-    QScopedArrayPointer<uchar> xMask(new uchar[height * n]);
+    // a ddb is word aligned, QImage depends on bow it was created
+    const auto bplDdb = qMax(1, ((width + 15) >> 4) << 1);
+    const auto bplImg = int(bbits.bytesPerLine());
+    QScopedArrayPointer<uchar> xBits(new uchar[height * bplDdb]);
+    QScopedArrayPointer<uchar> xMask(new uchar[height * bplDdb]);
     int x = 0;
     for (int i = 0; i < height; ++i) {
         const uchar *bits = bbits.constScanLine(i);
         const uchar *mask = mbits.constScanLine(i);
-        for (int j = 0; j < n; ++j) {
+        for (int j = 0; j < bplImg && j < bplDdb; ++j) {
             uchar b = bits[j];
             uchar m = mask[j];
             if (invb)
@@ -121,6 +125,11 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
             xMask[x] = b ^ m;
             ++x;
         }
+        for (int i = bplImg; i < bplDdb; ++i) {
+            xBits[x] = 0;
+            xMask[x] = 0;
+            ++x;
+        }
     }
     return CreateCursor(GetModuleHandle(nullptr), hotSpot.x(), hotSpot.y(), width, height,
                         xBits.data(), xMask.data());
@@ -138,8 +147,8 @@ static HCURSOR createBitmapCursor(const QCursor &cursor, qreal scaleFactor = 1)
         bbits = bbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
         mbits = mbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     }
-    bbits = bbits.convertToFormat(QImage::Format_Mono);
-    mbits = mbits.convertToFormat(QImage::Format_Mono);
+    bbits = std::move(bbits).convertToFormat(QImage::Format_Mono);
+    mbits = std::move(mbits).convertToFormat(QImage::Format_Mono);
     const bool invb = bbits.colorCount() > 1 && qGray(bbits.color(0)) < qGray(bbits.color(1));
     const bool invm = mbits.colorCount() > 1 && qGray(mbits.color(0)) < qGray(mbits.color(1));
     return createBitmapCursor(bbits, mbits, cursor.hotSpot(), invb, invm);
@@ -436,8 +445,8 @@ QWindowsCursor::PixmapCursor QWindowsCursor::customCursor(Qt::CursorShape cursor
     if (!bestFit)
         return PixmapCursor();
 
-    const QPixmap rawImage(QStringLiteral(":/qt-project.org/windows/cursors/images/") +
-                           QString::fromLatin1(bestFit->fileName));
+    const QPixmap rawImage(":/qt-project.org/windows/cursors/images/"_L1 +
+                           QLatin1StringView(bestFit->fileName));
     return PixmapCursor(rawImage, QPoint(bestFit->hotSpotX, bestFit->hotSpotY));
 }
 #endif // !QT_NO_IMAGEFORMAT_PNG
@@ -659,16 +668,16 @@ void QWindowsCursor::setPos(const QPoint &pos)
 */
 QSize QWindowsCursor::size() const
 {
-    const QPair<DWORD,bool> cursorSizeSetting =
+    const auto cursorSizeSetting =
         QWinRegistryKey(HKEY_CURRENT_USER, LR"(Control Panel\Cursors)")
-                       .dwordValue(L"CursorBaseSize");
+                       .value<DWORD>(L"CursorBaseSize");
     const int baseSize = screenCursorSize(m_screen).width() / 2;
-    if (!cursorSizeSetting.second)
+    if (!cursorSizeSetting)
         return QSize(baseSize / 2, baseSize / 2);
 
     // The registry values are dpi-independent, so we need to scale the result.
-    int cursorSizeValue = cursorSizeSetting.first * m_screen->logicalDpi().first
-                                                  / m_screen->logicalBaseDpi().first;
+    int cursorSizeValue = *cursorSizeSetting * m_screen->logicalDpi().first
+                                             / m_screen->logicalBaseDpi().first;
 
     // map from registry value 32-256 to 0-14, and from there to pixels
     cursorSizeValue = (cursorSizeValue - 2 * baseSize) / baseSize;
